Module adapter

Module adapter 

Named collections backed by Git commits and compare-and-swap refs.

Structs

Collection
A named collection view backed by the Git commit/ref adapter.
Database
A Git repository that manages named, mutable collection refs.