#[non_exhaustive]pub enum Error {
Git(Error),
Json(Error),
Io(Error),
Embedding(String),
Invalid(String),
CollectionNotFound(String),
CollectionExists(String),
StaleRoot {
expected: ObjectId,
actual: ObjectId,
},
ReadOnly,
Corrupt(String),
}Expand description
An error returned by a git-vdb operation.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Git(Error)
The underlying Git object database returned an error.
Json(Error)
JSON serialization or deserialization failed.
Io(Error)
A filesystem operation failed.
Embedding(String)
An embedding provider could not initialize or generate vectors.
Invalid(String)
The request or persisted value violated a declared invariant.
CollectionNotFound(String)
A requested named collection does not exist.
CollectionExists(String)
A collection could not be created because its name already exists.
StaleRoot
A named write expected a different current collection root.
Fields
ReadOnly
A write was attempted through an immutable historical collection view.
Corrupt(String)
Stored Git objects do not form a valid canonical collection root.