pub struct Filter {
pub must: Vec<Condition>,
pub should: Vec<Condition>,
pub must_not: Vec<Condition>,
}Expand description
A Boolean point filter.
Every must condition and no must_not condition must match. When should
is non-empty, at least one should condition must also match.
Fields
must: Vec<Condition>Conditions that must all match.
should: Vec<Condition>Conditions of which at least one must match when the list is non-empty.
must_not: Vec<Condition>Conditions that must not match.
Implementations
Sourceimpl Filter
impl Filter
Sourcepub fn must(conditions: impl IntoIterator<Item = Condition>) -> Self
pub fn must(conditions: impl IntoIterator<Item = Condition>) -> Self
Creates a filter containing only required conditions.
Examples found in repository?
More examples
examples/filtering.rs (line 13)
4fn main() -> git_vdb::Result<()> {
5 let temporary = tempfile::TempDir::new()?;
6 let docs = open(temporary.path().join("vectors.git"))?.collection("docs");
7 docs.upsert([
8 Point::new("guide", [1.0, 0.0]).with_metadata(json!({"kind": "docs"}))?,
9 Point::new("code", [0.9, 0.1]).with_metadata(json!({"kind": "source"}))?,
10 ])?;
11 let result = docs.advanced()?.query(
12 Query::new([1.0, 0.0], 5)
13 .with_filter(Filter::must([Condition::matches("kind", "docs")]))
14 .with_payload(),
15 )?;
16 assert_eq!(result.points[0].id.to_string(), "guide");
17 Ok(())
18}Trait Implementations
Sourceimpl<'de> Deserialize<'de> for Filter
impl<'de> Deserialize<'de> for Filter
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more