pub struct Filter {
pub must: Vec<Condition>,
pub should: Vec<Condition>,
pub must_not: Vec<Condition>,
}Expand description
A Boolean point filter.
Every must condition and no must_not condition must match. When should
is non-empty, at least one should condition must also match.
Fields
must: Vec<Condition>Conditions that must all match.
should: Vec<Condition>Conditions of which at least one must match when the list is non-empty.
must_not: Vec<Condition>Conditions that must not match.
Implementations