Provider-independent document embedding and text search.
Structs
- Document
- A text document with a typed ID and optional JSON metadata.
- Document
Hit - A typed document similarity-search result.
- Fast
Embedder - A local FastEmbed text model, available with the
fastembedfeature. - Text
Collection - A collection that embeds documents and text queries with one model.
- Text
Query - A text similarity query with optional filtering and execution controls.
Enums
Traits
- Embedder
- Converts text batches into vectors from one stable model space.
Type Aliases
- Fast
Embed Init Options - Options for initializing the TextEmbedding model